(define (domain first-response)
  (:requirements :typing 
		 :equality
		 :negative-preconditions
		 :disjunctive-preconditions
		 :universal-preconditions
		 :conditional-effects
		 :existential-preconditions
		 :probabilistic-effects)
 (:types location victim status fire_unit medical_unit)
 (:constants healthy hurt dying - status)
 (:predicates 
  (fire ?l - location)
  (rubble ?l1 ?l2 - location)
  (victim-at ?v - victim ?l - location)
  (victim-status ?v - victim ?s - status)
  (hospital ?l - location)
  (water-at ?l - location)
  (adjacent ?l1 ?l2 - location)
  (fire-unit-at ?u - fire_unit ?l - location)
  (medical-unit-at ?u - medical_unit ?l - location)
  (have-water ?u - fire_unit)
  (have-victim-in-unit ?v - victim ?u - medical_unit)
  )

 (:action drive-fire-unit
  :parameters (?u - fire_unit ?from - location ?to - location)
  :precondition (and (fire-unit-at ?u ?from)
		     (adjacent ?to ?from)
		     (not (fire ?to))
		     (not (rubble ?to ?from)))
  :effect (and (fire-unit-at ?u ?to) (not (fire-unit-at ?u ?from)))
  )

 (:action drive-medical-unit
  :parameters (?u - medical_unit ?from - location ?to - location)
  :precondition (and (medical-unit-at ?u ?from)
		     (adjacent ?to ?from)
		     (not (fire ?to))
		     (not (rubble ?to ?from)))
  :effect (and (medical-unit-at ?u ?to) (not (medical-unit-at ?u ?from)))
  )

 

 (:action load-fire-unit
  :parameters (?u - fire_unit ?l - location)
  :precondition (and (fire-unit-at ?u ?l))
  :effect (when (water-at ?l) (have-water ?u)))

 (:action load-medical-unit
  :parameters (?u - medical_unit ?l - location)
  :precondition (and (medical-unit-at ?u ?l))
  :effect (forall (?v - victim) 
		  (when (victim-at ?v ?l)
		        (and (have-victim-in-unit ?v ?u) 
			     (not (victim-at ?v ?l))))))

 
 (:action unload-fire-unit
  :parameters (?u - fire_unit ?l ?l1 - location)
  :precondition (and (fire-unit-at ?u ?l)
                     (or (= ?l1 ?l) (adjacent ?l1 ?l)))
  :effect (when (have-water ?u)
                (and (not (have-water ?u))
		     (when (fire ?l1)
			   (probabilistic 0.7 (not (fire ?l1)))))))

 (:action unload-medical-unit
  :parameters (?u - medical_unit ?l - location)
  :precondition (and (medical-unit-at ?u ?l))
  :effect (forall (?v - victim)
		  (when (have-victim-in-unit ?v ?u)
		        (and (victim-at ?v ?l) (not (have-victim-in-unit ?v ?u))))))


 (:action treat-victim-on-scene-medical
  :parameters (?u - medical_unit ?l - location ?v - victim)
  :precondition (and (medical-unit-at ?u ?l) 
		     (victim-at ?v ?l)
		     (victim-status ?v hurt))
  :effect (probabilistic 0.9 (and (victim-status ?v healthy) 
				  (not (victim-status ?v hurt)))))

 (:action treat-victim-on-scene-fire
  :parameters (?u - fire_unit ?l - location ?v - victim)
  :precondition (and (fire-unit-at ?u ?l) 
		     (victim-at ?v ?l)
		     (victim-status ?v hurt))
  :effect (probabilistic 0.9 (and (victim-status ?v healthy) 
				  (not (victim-status ?v hurt)))))

 (:action treat-victim-at-hospital
  :parameters (?v - victim ?l - location)
  :precondition (and (victim-at ?v ?l)
		     (hospital ?l))		     
  :effect (and (victim-status ?v healthy) 
	       (not (victim-status ?v hurt))
	       (not (victim-status ?v dying))))
  
)